///////////////////////////////////////////////////////////////////////////
/// Fichero: termometro.h                                   
/// Contenido: definicion de la clase termometro          
/// Autor: Nicolas Luis Fernandez Garca                   
/// Ultima modificacion: 8/diciembre/2009                    
///////////////////////////////////////////////////////////////////////////

#ifndef _TERMOMETRO_H_
#define _TERMOMETRO_H_

///////////////////////////////////////////////////////////////////////////
/// Clase: Termometro                                                  
/// Descripcion:                                                        
///               Representa un termometro con diferentes escalas:      
///                - celsius                                            
///                - fahrenheit                                         
///                - kelvin                                             
/// Representacion:                                                      
///          La temperatura se va a almacenar en grados celsius         
///          Se utilizaran funciones de conversion para obtener        
///          el valor de la temperatura en otras escalas:               
/// Herencia: no hereda de ninguna clase                                
///////////////////////////////////////////////////////////////////////////
 
 class Termometro
 {

          /// Sobrecargas del operador + usando funciones friend

          /// Operador binario "+" utilizado en expresiones t1 = 10 + t2;
          friend Termometro operator+ (double dato_grados_celsius, Termometro objeto_termometro);

          /// Operador binario "-" utilizado en expresiones t1 = 10 - t2;
          friend Termometro operator- (double dato_grados_celsius, Termometro objeto_termometro);


    public:

           /// Constructor sin parametros.
            Termometro();

           /// Constructor con un parametro.
            Termometro(double dato_grados_celsius);

	    /// Funciones de consulta

            /// Funcion de consulta de grados celsius
            double get_grados_celsius ();

            /// Funcion de consulta de grados fahrenheit
            double get_grados_fahrenheit ();

            /// Funcion de consulta de grados kelvin
            double get_grados_kelvin ();

	    /// Funciones de modificacion

            /// Funcion de modificacion de grados celsius
            void set_grados_celsius (double dato_grados_celsius);

            /// Funcion de modificacion de grados fahrenheit
            void set_grados_fahrenheit (double dato_grados_fahrenheit);

            /// Funcion de modificacion de grados kelvin
            void set_grados_kelvin (double dato_grados_kelvin);

	    /// Sobrecarga de operadores

            /// Operador binario "+" utilizado en expresiones t1 = t2 + 10;
            Termometro operator+ (double dato_grados_celsius);

            /// Operador binario "-" utilizado en expresiones t1 = t2 - 10;
            Termometro operator- (double dato_grados_celsius);

            /// Sobrecarga del operador de asignacion =
             Termometro & operator=(const Termometro &dato_termometro);
	   
            /// Operadores unarios de incremento

            /// Operador unario utilizado en t1 = t2++ (postfijo)
 	    /// Asigna a t1 y despues incrementa t2
	    Termometro operator++(int);

            /// Operador unario utilizado en t1 = ++t2 (prefijo)
	    /// Incrementa t2 y despues asigna a t1
	    Termometro operator++();

            /// Operadores unarios de decremento

            /// Operador unario utilizado en t1 = t2-- (postfijo)
 	    /// Asigna a t1 y despues decrementa t2
	    Termometro operator--(int);

            /// Operador unario utilizado en t1 = --t2 (prefijo)
	    /// Decrementa t2 y despues asigna a t1
	    Termometro operator--();

            /// Operadores binarios de incremento (decremento) y asignacion 

            /// Operador binario utilizado en t1 += 10 
	    /// Asigna a t1 el valor de t1 + 10
	     Termometro operator+=(double dato_grados_celsius);

            /// Operador binario utilizado en t1 -= 10 
            /// Asigna a t1 el valor de t1 - 10
	    Termometro operator-=(double dato_grados_celsius);

	    /// Operadores relacionales

            /// Operador binario de igualdad t1 == t2
	     bool operator==(const Termometro &dato_termometro);

	     /// Operador binario de desigualdad t1 != t2
 	     bool operator!=(const Termometro &dato_termometro);

	     /// Operador binario menor que <
	     bool operator<(const Termometro &dato_termometro);

	     /// Operador binario menor o igual que <=
	     bool operator<=(const Termometro &dato_termometro);

	     /// Operador binario mayor que >
	     bool operator>(const Termometro &dato_termometro);
 
	     /// Operador binario de igualdad >=
	     bool operator>=(const Termometro &dato_termometro);


	 	 
    private:
            /// Campo o atributo en el que va a almacenar la temperatura
	    double _grados_celsius; 


         /// funciones privadas de conversion de grados

            double convertir_grados_celsius_a_fahrenheit (double dato_grados_celsius);
            double convertir_grados_fahrenheit_a_celsius (double dato_grados_fahrenheit);

            double convertir_grados_celsius_a_kelvin  (double dato_grados_celsius);
	    double convertir_grados_kelvin_a_celsius (double dato_grados_kelvin);
            
 };
 
#endif


